package asd.td2;

import asd.base.*;

/**
 * Une classe pour l'exercice 4 de la feuille 2.
 *  2002 Marc Gatano
 */
public class PairesSuite {

    private Suite s;        // une suite strictement croissante d'entiers
    private FileStatique f; // la file

    /**
     * Construit un objet PairesSuite.
     * Prcondition : suite est une suite strictement croissante d'entiers.
     */
    public PairesSuite(Suite suite) {
	s = suite;
    }
    
    /**
     * Affiche toutes les paires d'entier x et y de la suite, tels que
     * x + n = y.
     * Precondition : la suite est strictement croissante et n > 0.
     * Complexite : O(n) ou n est le nombre d'elements de la suite.
     */
    public void afficherLesPaires(int n) {

        f = new FileStatique(n);

	s.debut();

        // INVARIANT :
        // ( s = G.D et s.valeurCourante() est le premier element de D )
        // Les paires (x,y) de G telles que x + n = y ont ete imprimees.
        // si ! f.vide() alors f = -> zk,zk-1,..,z1 -> ou  z1,..,zk-1,zk
        // est la suite croissante de tous les elements zi de G tels que
        // zk < zi + n.
        // VARIANT : |G| croit et est borne par |s|.
        //
        while ( ! s.fin() ) {

            int v = s.valeurCourante();

            // VARIANT : f.longueur() decroit et est borne par 0
            //
            while ( ! vide() && v > suivant() + n ) {
                retirer();
            }
		
            if ( ! vide() && v == suivant() + n ) {
                afficher(suivant(),v);
                retirer();
            }

            ajouter(v);
            s.suivant();
        }
    }

    // affiche la paire d'entier (x,y)
    //
    private void afficher(int x, int y) {
	System.out.println("(" + x + ", " + y + ")");
    }

    // pour se simplifier la vie

    private void ajouter(int n) {
        f.ajouter(new Integer(n));
    }

    private void retirer() {
        f.retirer();
    }

    private int suivant() {
        return ((Integer) f.suivant()).intValue();
    }

    private boolean vide() {
        return f.vide();
    }
}
